@@ -85,6 +85,11 @@ str.emmGrid <- function(object, ...) {
         cat(paste(tmp, collapse = ", "))
         cat("\n")
     }
+    if (length(cf <- object@roles$counterfactuals) > 0) {
+        cat("Counterfactuals\n    ")
+        cat(paste(cf, collapse = ", "))
+        cat("\n")
+    }
     if(!is.null(object@model.info$nesting)) {
         cat("Nesting structure:  ")
         cat(.fmt.nest(object@model.info$nesting))
@@ -113,9 +118,16 @@ str.emmGrid <- function(object, ...) {
 #' @method print emmGrid
 #' @param x An \code{emmGrid} object
 #' @export
-print.emmGrid = function(x, ..., export = FALSE)
-    print(summary.emmGrid(x, ...), export = export)
+print.emmGrid = function(x, ..., export = FALSE) {
+    if(export) 
+        print(summary.emmGrid(x, ...), export = export)
+    else
+        show(x)
+}
 
+### Former print method which I changed to work around a bug in pkgdown
+# print.emmGrid = function(x, ..., export = FALSE)
+#     print(summary.emmGrid(x, ...), export = export)
 
 # vcov method
 #' Miscellaneous methods for \code{emmGrid} objects
@@ -332,6 +344,11 @@ vcov.emmGrid = function(object, ..., sep = get_emm_option("sep")) {
 #' situation such as an \code{lm} model, the joint test
 #' of the modified object is in essence a type-2 test as in \code{car::Anova}.
 #' 
+#' Please note that it is possible (or even likely) that there will be disparity
+#' between the \code{grid} and \code{linfct} slots when a submodel is used. This is
+#' because \code{grid} contains the \emph{claimed} values of the predictors and
+#' \code{linfct} contains \emph{aliases} of them computed from the submodel.
+#' 
 #' For some objects such as generalized linear models, specifying \code{submodel}
 #' will typically not produce the same estimates or type-2 tests as would be
 #' obtained by actually fitting a separate model with those specifications.
@@ -505,6 +522,10 @@ update.emmGrid = function(object, ..., silent = FALSE) {
 #'   \code{\link{summary.emmGrid}}, \code{\link{predict.emmGrid}}, \code{\link{test.emmGrid}},
 #'   \code{\link{confint.emmGrid}}, and \code{\link{emmip}}. The only option that can
 #'   affect the latter four is \code{"predict.method"}.}
+#' \item{\code{allow.na.levs}}{A logical value that if \code{TRUE} (the default), allows
+#'   \code{NA} to be among the levels of a factor. Older versions of \pkg{emmeans} did not
+#'   allow this. So if problems come up (say in a messy dataset that includes incomplete cases), 
+#'   try setting this to \code{FALSE}.}
 #' \item{\code{sep}}{A character value to use as a separator in labeling factor combinations.
 #'   Such labels are potentially used in several places such as \code{\link{contrast}} and 
 #'   \code{\link{plot.emmGrid}} when combinations of factors are compared or plotted.
@@ -672,6 +693,34 @@ get_emm_option = function(x, default = emm_defaults[[x]]) {
         default
 }
 
+#' @rdname emm_options
+#' @param expr Expression to evaluate. If missing, the last element of \code{...} is used.
+#' @return \code{with_emm_options()} temporarily sets the options in \code{...}, then 
+#'   evaluates \code{try(expr)} and returns the result.
+#' @export
+#' @examples
+#' 
+#' # Illustration of how 'opt.digits' affects the results of print()
+#' # Note that the returned value is printed with the default setting (opt.digits = TRUE)
+#' pigs.lm <- lm(inverse(conc) ~ source, data = pigs)
+#' with_emm_options(opt.digits = FALSE, print(emmeans(pigs.lm, "source")))
+#' 
+with_emm_options = function(..., expr) {
+    cl = match.call()
+    if(missing(expr)) {
+        expr = cl[[length(cl)]]
+        cl = cl[-length(cl)]
+    }
+    cl[[1]] = as.name("emm_options")
+    oldopts = getOption("emmeans")
+    eval(cl, parent.frame())
+    result = try(eval(expr, parent.frame()))
+    options(emmeans = oldopts)
+    result
+}
+
+
+
 ### Exported defaults for certain options
 #' @rdname emm_options
 #' @export
@@ -680,6 +729,7 @@ emm_defaults = list (
     emmeans = list(infer = c(TRUE, FALSE)),
     contrast = list(infer = c(FALSE, TRUE)),
     save.ref_grid = FALSE,    # save new ref_grid in .Last.ref_grid
+    allow.na.levs = TRUE,     # Do we allow factors to have NA as a level?
     cov.keep = "2",           # default for cov.keep arg in ref_grid
     sep = " ",                # separator for combining factor levels
     parens = c("-|\\+|\\/|\\*", "(", ")"), # patterns for what/how to parenthesize in contrast
@@ -742,116 +792,6 @@ emm_defaults = list (
     update.emmGrid(x, levels = value)
 }
 
-# ### transform method (I decided to ditch this completely in favor of levels<-)
-# #' Modify variable names and/or levels in a reference grid
-# #'
-# #' @param `_data` An object of class \code{emmGrid}
-# #' @param ... Specifications for changes to be made. See Specifications section
-# #' @param `_par` named \code{list} containing any additional parameters needed
-# #' in evaluating expressions
-# #' 
-# #' @section Specifications:
-# #' Each specification can be of one of the following forms:
-# #' \itemize{
-# #'   \item{\code{name = <replacement levels>  } (replace levels only)}
-# #'   \item{\code{name = newname ~ <replacement levels>  } (replace levels and rename)}
-# #'   \item{\code{name = ~ <expression>  } (calculate new levels)}
-# #'   \item{\code{name = newname ~ <expression>  } (calculate new levels and rename)}
-# #'   \item{\code{newame ~ name  } (rename with levels unchanged)}
-# #'   \item{\code{newname ~ <expression>  } (calculate new levels for variable 
-# #'          in expression, and rename)}
-# #' }
-# #' Here, \code{name} must be the name of an existing predictor in the grid,
-# #' and \code{<replacement levels>} is a character of numeric vector of length
-# #' exactly equal to the number of levels of \code{name}. The type of the replacement levels
-# #' does not need to match the type of the existing levels; however, any factor in the grid
-# #' remains a factor, with its levels changed.
-# #' 
-# #' Expressions must be supplied via a formula, and must be evaluable in the context 
-# #' of \code{envir} and the existing levels of \code{name}.
-# #' If a formula has a left-hand side, it is used as
-# #' a replacement name for that variable.
-# #'
-# #' @return a modified \code{emmGrid} object
-# #' @export
-# #' 
-# #' @note
-# #' An alternative way to use this is to supply a list of arguments as the \code{morph}
-# #' option in \code{\link{update.emmGrid}}.
-# #'
-# #' @examples
-# #' warp.lm <- lm(breaks ~ wool * tension, data = warpbreaks)
-# #' (warp.rg <- ref_grid(warp.lm))
-# #' transform(warp.rg, tension = 1:3,  wool = texture ~ c("soft", "coarse"))
-# #' 
-# #' # Standardized predictor
-# #' z <- scale(fiber$diameter)
-# #' fiber.lm <- lm(strength ~ z + machine, data = fiber)
-# #' 
-# #' ### Mean predictions at 1-SD intervals:
-# #' (fiber.emm <- emmeans(fiber.lm, "z", at = list(z = -1:1)))
-# #' 
-# #' ### Same predictions labeled with actual diameter values:
-# #' transform(fiber.emm, diameter ~ `scaled:center` + `scaled:scale` * z,
-# #'     `_par` = attributes(z))
-# #' 
-# #' 
-# 
-# transform.emmGrid <- function(`_data`, ..., `_par` = list()) {
-#     specs = list(...)
-#     nms = names(c(`_dummy_` = 0, specs))[-1]  # keeps this from being NULL
-#     for (i in which(nms == "")) { # get name from formula rhs
-#         if (!inherits(spc <- specs[[i]], "formula") 
-#             || (inherits(spc, "formula") && (length(spc) < 3)))
-#             stop("Unnamed specifications must be two-sided formulas")
-#         nms[i] = names(specs)[i] = c(intersect(all.vars(spc[-2]),
-#                         names(`_data`@levels)), "(absent)")[1]
-#     }
-#     for (var in nms) {
-#         oldlev = `_data`@levels[[var]]
-#         if (is.null(oldlev))
-#             stop("No variable named '", var, "' in this object.")
-#         newlev = specs[[var]]
-#         if(inherits(newlev, "formula") && length(newlev) > 2) {
-#             newname = as.character(newlev)[2]
-#             newlev = newlev[-2]
-#         }
-#         else
-#             newname = ""
-#         if ((is.numeric(newlev) || is.character(newlev)) 
-#             && length(newlev) != length(oldlev))
-#             stop("Must provide exactly ", length(oldlev), " levels for '", var, "'")
-#         else if (inherits(newlev, "formula"))
-#             newlev = eval(str2expression(as.character(newlev)[2]), 
-#                           envir = c(`_data`@levels[var], `_par`))
-#         # so at this point we have conforming numbers of levels.
-#         `_data`@levels[[var]] = newlev
-#         v = newv = `_data`@grid[[var]]
-#         if (inherits(v, "factor"))
-#             levels(newv) = newlev
-#         else for (i in seq_along(oldlev)) 
-#             newv[v == oldlev[[i]]] = newlev[[i]]
-#         `_data`@grid[[var]] = newv
-#         if (newname != "") {
-#             i = which(names(`_data`@levels) == var)
-#             names(`_data`@levels)[i] = newname
-#             i = which(names(`_data`@grid) == var)
-#             names(`_data`@grid)[i] = newname
-#             i = which(`_data`@roles$predictors == var)
-#             if (length(i) > 0) 
-#                 `_data`@roles$predictors[i] = newname
-#             nst = `_data`@model.info$nesting
-#             if (!is.null(nst)) {
-#                 names(nst)[names(nst) == var] = newname
-#                 nst = lapply(nst, function(x) { x[x == var] = newname; x })
-#                 `_data`@model.info$nesting = nst
-#             }
-#         }
-#     }
-#     `_data`
-# }
-
-
 
 ### Utility to change the internal structure of an emmGrid object
 ### Returned emmGrid object has linfct = I and bhat = estimates
@@ -983,7 +923,8 @@ regrid = function(object, transform = c("response", "mu", "unlink", "none", "pas
     
     if (is.na(object@post.beta[1]) && !missing(N.sim)) {
         message("Simulating a sample of size ", N.sim, " of regression coefficients.")
-        object@post.beta = sim(N.sim, object@bhat, object@V)
+        bh = object@bhat[!is.na(object@bhat)]
+        object@post.beta = sim(N.sim, bh, object@V)
     }
 
     if (transform == "pass")
@@ -993,64 +934,58 @@ regrid = function(object, transform = c("response", "mu", "unlink", "none", "pas
     if ((transform == "response") && (!is.null(object@misc$tran2)))
              object = regrid(object, transform = "mu")
 
-    .collapse = (\(.collapse = NULL, ...) .collapse)(...) # check counterfactuals hook
-    
     # Save post.beta stuff
     PB = object@post.beta
     NC = attr(PB, "n.chains")
     
     if (!is.na(PB[1])) { # fix up post.beta BEFORE we overwrite parameters
-        PB = PB %*% t(object@linfct)
+        PB = PB %*% t(object@linfct[ , !is.na(object@bhat), drop = FALSE])
         if (".offset." %in% names(object@grid))
             PB = t(apply(PB, 1, function(.) . + object@grid[[".offset."]]))
     }
     
     est = .est.se.df(object, do.se = TRUE) ###FALSE)
     
-    if(is.null(.collapse)) {
-        estble = !(is.na(est[[1]]))
-        object@V = vcov(object)[estble, estble, drop = FALSE]
-        object@bhat = est[[1]]
-        object@linfct = diag(1, length(estble))
-        pargs = object@grid[names(object@levels)]
-        lbls = do.call(paste, c(pargs, sep = "."))
-        if (!is.null(disp <- object@misc$display)) {  # fix up for the bookkeeping in nested models
-            object@V = object@V[estble, estble, drop = FALSE]
-            object@linfct = matrix(0, nrow = length(disp), ncol = length(estble))
-            object@linfct[disp, ] = diag(1, length(estble))
-            lbls = lbls[disp]
-        }
-        colnames(object@linfct) = lbls
-        if(all(estble))
-            object@nbasis = estimability::all.estble
-        else
-            object@nbasis = object@linfct[, !estble, drop = FALSE]
-        
-        # override the df function
-        df = est$df
-        edf = df[estble]
-        if (length(edf) == 0) edf = NA
-        # note both NA/NA and Inf/Inf test is.na() = TRUE
-        prev.df.msg = attr(object@dffun, "mesg")
-        if (any(is.na(edf/edf)) || (diff(range(edf)) < .01)) { # use common value
-            object@dfargs = list(df = mean(edf, na.rm = TRUE))
-            object@dffun = function(k, dfargs) dfargs$df
-        }
-        else { # use containment df
-            object@dfargs = list(df = df)
-            object@dffun = function(k, dfargs) {
-                idx = which(zapsmall(k) != 0)
-                ifelse(length(idx) == 0, NA, min(dfargs$df[idx], na.rm = TRUE))
-            }
-        }
-        if(!is.null(prev.df.msg)) 
-            attr(object@dffun, "mesg") = ifelse(
-                startsWith(prev.df.msg, "inherited"), prev.df.msg,
-                paste("inherited from", prev.df.msg, "when re-gridding"))
+    estble = !(is.na(est[[1]]))
+    object@V = vcov(object)[estble, estble, drop = FALSE]
+    object@bhat = est[[1]]
+    object@linfct = diag(1, length(estble))
+    object@misc$regrid.flag = TRUE
+    pargs = object@grid[names(object@levels)]
+    lbls = do.call(paste, c(pargs, sep = "."))
+    if (!is.null(disp <- object@misc$display)) {  # fix up for the bookkeeping in nested models
+        object@V = object@V[estble, estble, drop = FALSE]
+        object@linfct = matrix(0, nrow = length(disp), ncol = length(estble))
+        object@linfct[disp, ] = diag(1, length(estble))
+        lbls = lbls[disp]
     }
+    colnames(object@linfct) = lbls
+    if(all(estble))
+        object@nbasis = estimability::all.estble
+    else
+        object@nbasis = object@linfct[, !estble, drop = FALSE]
     
-    if(!is.null(.collapse) && is.null(object@misc$tran))  # need explicit link so we can collapse
-        object@misc$tran = attr(est, "link") = make.link("identity")
+    # override the df function
+    df = est$df
+    edf = df[estble]
+    if (length(edf) == 0) edf = NA
+    # note both NA/NA and Inf/Inf test is.na() = TRUE
+    prev.df.msg = attr(object@dffun, "mesg")
+    if (any(is.na(edf/edf)) || (diff(range(edf)) < .01)) { # use common value
+        object@dfargs = list(df = mean(edf, na.rm = TRUE))
+        object@dffun = function(k, dfargs) dfargs$df
+    }
+    else { # use containment df
+        object@dfargs = list(df = df)
+        object@dffun = function(k, dfargs) {
+            idx = which(zapsmall(k) != 0)
+            ifelse(length(idx) == 0, NA, min(dfargs$df[idx], na.rm = TRUE))
+        }
+    }
+    if(!is.null(prev.df.msg)) 
+        attr(object@dffun, "mesg") = ifelse(
+            startsWith(prev.df.msg, "inherited"), prev.df.msg,
+            paste("inherited from", prev.df.msg, "when re-gridding"))
     
     if(transform %in% c("response", "mu", "unlink", links, "user") && !is.null(object@misc$tran)) {
         flink = link = attr(est, "link")
@@ -1066,35 +1001,11 @@ regrid = function(object, transform = c("response", "mu", "unlink", "none", "pas
         }
         if (!is.na(PB[1]))
             PB = matrix(flink$linkinv(PB), ncol = ncol(PB))
-        if(is.null(.collapse)) {
-            D = flink$mu.eta(object@bhat[estble])
-            object@bhat = flink$linkinv(object@bhat)
-            # efficient repl for D'VD with D <- diag(D)
-            object@V = sweep(sweep(object@V, 1, D, "*"), 2, D, "*")
-        }
-        else {  # we'll average over the levels of .collapse (assume it varies slowest)
-            est = est[[1]]
-            nobs = length(object@levels[[.collapse]])
-            idx = sapply(seq_len(nobs), \(i) which(object@grid[[.collapse]] == i))
-            X = sweep(object@linfct, 1, flink$mu.eta(est), "*")
-            wt.counter = (\(wt.counter, ...) wt.counter)(...)
-            if (length(wt.counter) == 1) wt.counter = rep(1, nobs)
-            wmn = function(x) sum(wt.counter*x) / sum(wt.counter)
-            est = flink$linkinv(est)
-            object@bhat = sapply(seq_len(nrow(idx)), \(i) wmn(est[idx[i,]]))
-            L = matrix(0, nrow = nrow(idx), ncol = ncol(X))
-            for (i in seq_len(nrow(idx)))
-                L[i, ] = apply(X[idx[i,], , drop = FALSE], 2, wmn)
-            object@V = L %*% tcrossprod(object@V, L)
-            object@linfct = diag(1, nrow(L))
-            if(!is.na(PB[1])) {
-                pb = matrix(0, ncol = nrow(L), nrow = nrow(PB))
-                for (i in seq_len(nrow(idx)))
-                    pb[, i] = apply(PB[, idx[i,], drop = FALSE], 1, wmn)
-                PB = pb
-            }
-        }
-        
+        D = flink$mu.eta(object@bhat[estble])
+        object@bhat = flink$linkinv(object@bhat)
+        # efficient repl for D'VD with D <- diag(D)
+        object@V = sweep(sweep(object@V, 1, D, "*"), 2, D, "*")
+
         inm = object@misc$inv.lbl
         if (!is.null(inm)) {
             object@misc$estName = inm
@@ -1142,6 +1053,7 @@ regrid = function(object, transform = c("response", "mu", "unlink", "none", "pas
     }
     
     if(!is.na(PB[1])) {
+        PB = PB[ , !is.na(object@bhat), drop = FALSE]
         attr(PB, "n.chains") = NC
         object@post.beta = PB
     }
@@ -1152,13 +1064,7 @@ regrid = function(object, transform = c("response", "mu", "unlink", "none", "pas
     object@model.info$model.matrix = "Submodels are not available with regridded objects"
     if(!missing(predict.type))
         object = update(object, predict.type = predict.type)
-    if(!is.null(.collapse)) {
-        object@grid = object@grid[idx[,1], , drop = TRUE]
-        object@grid[[.collapse]] = object@levels[[.collapse]] = NULL
-        object@roles$predictors = setdiff(object@roles$predictors, .collapse)
-        object@misc$famSize = nrow(object@linfct)
-    }
-    
+
     object
 }
 
