@@ -27,12 +27,12 @@
 ### lm objects (and also aov, rlm, others that inherit) -- but NOT aovList
 ### Recent additional arhument 'frame' should point to where the model frame 
 ###   might be available, or NULL otherwise
-#' @method recover_data lm
+#' @exportS3Method recover_data lm
 #' @export
 recover_data.lm = function(object, frame = object$model, ...) {
         fcall = object$call
     recover_data(fcall, delete.response(terms(object)), object$na.action, 
-                 frame = frame, ...)
+                 frame = frame, pwts = weights(object), ...)
 }
 
 #' @export
@@ -89,6 +89,7 @@ emm_basis.mlm = function(object, trms, xlev, grid, ...) {
 
 #----------------------------------------------------------
 # manova objects
+#' @exportS3Method recover_data manova
 recover_data.manova = function(object, ...) {
     fcall = match.call(aov, object$call)   # need to borrow arg matching from aov()
     recover_data(fcall, delete.response(terms(object)), object$na.action, 
@@ -106,7 +107,7 @@ recover_data.merMod = function(object, ...) {
         return("Can't handle a nonlinear mixed model")
     fcall = object@call
     recover_data(fcall, delete.response(terms(object)), 
-                 attr(object@frame, "na.action"), frame = object@frame, ...)
+                 attr(object@frame, "na.action"), frame = object@frame, pwts = weights(object), ...)
 }
 
 #' @export
@@ -240,7 +241,7 @@ emm_basis.merMod = function(object, trms, xlev, grid,
 #' @export
 recover_data.lme = function(object, data, ...) {
     fcall = object$call
-    if (!is.null(eval(fcall$weights))) {  # painful -- we only get weights for complete cases
+    if (!is.null(eval(fcall$weights, environment(terms(object)), parent.frame()))) {  # painful -- we only get weights for complete cases
         if (!is.null(object$na.action)) {
             w = nlme::varWeights(object$modelStruct)
             wts = rep(0, length(w) + length(object$na.action))
@@ -339,6 +340,12 @@ gradV.kludge = function(object, Vname = "varFix", call = formula(object$terms),
     #     stop()
     # }
     
+    if(is.null(data)) {
+        vars = all.vars(eval(object$call[[2]]))
+        lst = lapply(vars, get)
+        names(lst) = vars
+        data = data.frame(lst)
+    }
     A = object$apVar
     theta = attr(A, "Pars")
     V = object[[Vname]]
@@ -348,10 +355,24 @@ gradV.kludge = function(object, Vname = "varFix", call = formula(object$terms),
     y = data[[yname]]
     n = length(y)
     object$call[[2]] = call
-    dat = t(replicate(2 + extra.iter + length(theta), {
-        data[[yname]] = y + sig * rnorm(n)
-        mod = update(object, data = data)
-        c(attr(mod$apVar, "Pars") - theta, as.numeric(mod[[Vname]] - V))
+    # we're gonna carefully check simulation results to make sure we didn't
+    # lose (or gain?) any params so that everything conforms
+    niter = 0
+    nsim = 2 + extra.iter + length(theta)
+    dat = t(replicate(nsim, {
+        simt = simv = numeric(0)
+        niter = niter + 1
+        while((niter < 3 * nsim) && 
+              ((length(simv) != length(V)) || (length(simt) != length(theta)))) {
+            data[[yname]] = y + sig * rnorm(n)
+            mod = update(object, data = data)
+            simt = attr(mod$apVar, "Pars")
+            simv = mod[[Vname]]
+        }
+        niter = niter + 1
+        if (niter > 3 * nsim) 
+            stop("Too many simulations due to inconsistency")
+        c(simt - theta, as.numeric(simv - V))
     }))
     dimnames(dat) = c(NULL, NULL)
     xcols = seq_along(theta)
@@ -389,11 +410,13 @@ gradV.kludge = function(object, Vname = "varFix", call = formula(object$terms),
 
 ### new way to get jacobians for gls models
 gls_grad = function(object, call, data, V) {
+    if (is.null(data)) 
+        data = environment(eval(call$model))
     obj = object$modelStruct
     conLin = object
     class(conLin) = class(obj)
     X = model.matrix(eval(call$model), data = data)
-    y = data[[all.vars(call)[1]]]
+    y = eval(eval(call$model)[[2]], envir = data)
     conLin$Xy = cbind(X, y)
     conLin$fixedSigma = FALSE
     func = function(x) {
@@ -408,6 +431,7 @@ gls_grad = function(object, call, data, V) {
 }
 
 ### gls objects (nlme package)
+#' @exportS3Method recover_data gls
 recover_data.gls = function(object, data, ...) {
     fcall = object$call
     if (!is.null(wts <- fcall$weights)) {
@@ -427,6 +451,7 @@ recover_data.gls = function(object, data, ...) {
     result
 }
 
+#' @exportS3Method emm_basis gls          
 emm_basis.gls = function(object, trms, xlev, grid, 
                          mode = c("auto", "df.error", "satterthwaite", "appx-satterthwaite", "boot-satterthwaite", "asymptotic"), 
                          extra.iter = 0, options, misc, ...) {
@@ -503,9 +528,11 @@ emm_basis.gls = function(object, trms, xlev, grid,
 
 #--------------------------------------------------------------
 ### polr objects (MASS package)
+#' @exportS3Method recover_data polr
 recover_data.polr = function(object, ...)
-    recover_data.lm(object, ...)
+    recover_data.clm(object, ...)
 
+#' @exportS3Method emm_basis polr         
 emm_basis.polr = function(object, trms, xlev, grid, 
                           mode = c("latent", "linear.predictor", "cum.prob", "exc.prob", "prob", "mean.class"), 
                           rescale = c(0,1), ...) {
@@ -550,20 +577,22 @@ emm_basis.polr = function(object, trms, xlev, grid,
 
 #--------------------------------------------------------------
 ### survreg objects (survival package)
+#' @exportS3Method recover_data survreg
 recover_data.survreg = function(object, ...) {
     fcall = object$call
     trms = delete.response(terms(object))
-    # I'm gonna delete any terms involving cluster(), or frailty() -- keep strata()
+    # I'm gonna delete any terms involving cluster(), frailty(); keep strata()
     mod.elts = dimnames(attr(trms, "factor"))[[2]]
-    tmp = grep("cluster\\(|frailty\\(", mod.elts)
+    tmp = grep("cluster\\(|frailty", mod.elts)
     if (length(tmp))
         trms = trms[-tmp]
-    recover_data(fcall, trms, object$na.action, ...)
+    recover_data(fcall, trms, object$na.action, pwts = weights(object), ...)
 }
 
 # Seems to work right in a little testing.
 # However, it fails sometimes if I update the model 
 # with a subset argument. Workaround: just fitting a new model
+#' @exportS3Method emm_basis survreg      
 emm_basis.survreg = function(object, trms, xlev, grid, ...) {
     # Much of this code is adapted from predict.survreg
     bhat = object$coefficients
@@ -578,7 +607,8 @@ emm_basis.survreg = function(object, trms, xlev, grid, ...) {
     X = model.matrix(trms, m, contrasts.arg = object$contrasts)
     usecols = intersect(colnames(X), names(bhat))
     bhat = bhat[usecols]  # in case ref_grid code excluded some levels...
-    nbasis = estimability::nonest.basis(model.matrix(object))
+    X = X[, usecols, drop=FALSE]
+    nbasis = estimability::nonest.basis(model.matrix(object)[, usecols, drop = FALSE])
     dfargs = list(df = object$df.residual)
     dffun = function(k, dfargs) dfargs$df
     if (object$dist %in% c("exponential","weibull","loglogistic","loggaussian","lognormal")) 
@@ -593,9 +623,11 @@ emm_basis.survreg = function(object, trms, xlev, grid, ...) {
 
 #--------------------------------------------------------------
 ###  coxph objects (survival package)
+#' @exportS3Method recover_data coxph
 recover_data.coxph = function(object, ...) 
     recover_data.survreg(object, ...)
 
+#' @exportS3Method emm_basis coxph        
 emm_basis.coxph = function (object, trms, xlev, grid, ...) 
 {
     object$dist = "doesn't matter"
@@ -604,7 +636,8 @@ emm_basis.coxph = function (object, trms, xlev, grid, ...)
     nms = colnames(result$X)
     # delete columns for intercept and main effects of strata
     zaps = which(nms %in% setdiff(nms, names(result$bhat)))
-    result$X = result$X[, -zaps, drop = FALSE]
+    if(length(zaps) > 0)
+        result$X = result$X[, -zaps, drop = FALSE]
     ### result$X = result$X - rep(object$means, each = nrow(result$X))
     result$misc$tran = "log"
     result$misc$inv.lbl = "hazard"
@@ -624,9 +657,11 @@ emm_basis.coxph = function (object, trms, xlev, grid, ...)
 #--------------------------------------------------------------
 ###  coxme objects ####
 ### Greatly revised 6-15-15 (after version 2.18)
+#' @exportS3Method recover_data coxme
 recover_data.coxme = function(object, ...) 
     recover_data.survreg(object, ...)
 
+#' @exportS3Method emm_basis coxme        
 emm_basis.coxme = function(object, trms, xlev, grid, ...) {
     bhat = coxme::fixef(object)
     k = length(bhat)
@@ -694,17 +729,21 @@ emm_basis.coxme = function(object, trms, xlev, grid, ...) {
 ###  gee objects  ####
 
 
+#' @exportS3Method recover_data gee
 recover_data.gee = function(object, ...)
     recover_data.lm(object, frame = NULL, ...)
 
+#' @exportS3Method emm_basis gee          
 emm_basis.gee = function(object, trms, xlev, grid, vcov.method = "robust.variance", ...)
     .emmb.geeGP(object, trms, xlev, grid, vcov.method, 
                 valid = c("robust.variance", "naive.variance"))
 
 ###  geepack objects  ####
+#' @exportS3Method recover_data geeglm
 recover_data.geeglm = function(object, ...)
     recover_data.lm(object, ...)
 
+#' @exportS3Method emm_basis geeglm       
 emm_basis.geeglm = function(object, trms, xlev, grid, vcov.method = "vbeta", ...) {
     m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
     X = model.matrix(trms, m, contrasts.arg = object$contrasts)
@@ -720,12 +759,13 @@ emm_basis.geeglm = function(object, trms, xlev, grid, vcov.method = "vbeta", ...
     
     misc = .std.link.labels(object$family, list())
     misc$initMesg = attr(V, "methMesg")
-    dffun = function(k, dfargs) Inf
-    dfargs = list()
+    dffun = function(k, dfargs) dfargs$df
+    dfargs = list(df = df.residual(object))
     list(X=X, bhat=bhat, nbasis=nbasis, V=V, dffun=dffun, dfargs=dfargs, misc=misc)
 }
 
 
+#' @exportS3Method recover_data geese
 recover_data.geese = function(object, ...) {
     fcall = object$call
     # what a pain - we need to reconstruct the terms component
@@ -741,6 +781,7 @@ recover_data.geese = function(object, ...) {
     recover_data(fcall, delete.response(trms), na.action, ...)
 }
 
+#' @exportS3Method emm_basis geese        
 emm_basis.geese = function(object, trms, xlev, grid, vcov.method = "vbeta", ...) {
     m = model.frame(trms, grid, na.action = na.pass, xlev = xlev)
     X = model.matrix(trms, m, contrasts.arg = object$contrasts)
@@ -766,8 +807,31 @@ emm_basis.geese = function(object, trms, xlev, grid, vcov.method = "vbeta", ...)
 }
 
 
-### survey pacvkage
+### glmtoolbox package -- we just pretend it's lm and it seems to work
+
+#' @exportS3Method recover_data glmgee
+recover_data.glmgee = function(object, ...) {
+    recover_data.lm(object, ...)
+}
+
+#' @exportS3Method emm_basis glmgee
+emm_basis.glmgee = function(object, trms, xlev, grid, vcov.method = "robust", ...) {
+    vcov. = if(is.character(vcov.method))
+        vcov(object, type = vcov.method, ...)
+    else
+        vcov.method
+    class(object) = c("glm", "lm") # pretend it's glm
+    rtn = emm_basis(object, trms = trms, xlev = xlev, grid = grid, vcov. = vcov., ...)
+    rtn$misc$initMesg = paste("Covariance estimate used:",
+        ifelse(is.character(vcov.method), vcov.method, "user-supplied"))
+    rtn
+}
+
+
+
+### survey package
 # svyglm class
+#' @exportS3Method recover_data svyglm
 recover_data.svyglm = function(object, data = NULL, ...) {
     if (is.null(data)) {
         env = environment(terms(object))
@@ -788,6 +852,13 @@ recover_data.svyglm = function(object, data = NULL, ...) {
 .statsvcov = function(object, ...)
     stats::vcov(object, complete = FALSE, ...)
 
+#' @rdname extending-emmeans
+#' @order 36
+#' @param vcov. Function or matrix that returns a suitable covariance matrix.
+#' The default is \code{.statsvcov} which is \code{stats::vcov}. The \code{.my.vcov} 
+#' function should be called in place of \code{\link{vcov}}, and it supports the user 
+#' being able to specify a different matrix or function via the
+#' optional \code{vcov.} argument.
 #' @export
 .my.vcov = function(object, vcov. = .statsvcov, ...) {
     if (is.function(vcov.))
@@ -798,8 +869,12 @@ recover_data.svyglm = function(object, data = NULL, ...) {
     vcov.
 }
 
-# Call this to do the standard stuff with link labels
-# Returns a modified misc
+#' @rdname extending-emmeans
+#' @order 30
+#' @param fam Result of call to \code{family(object)}
+#' @param misc A \code{list} intended for the \code{@misc} slot of an \code{emmGrid} object
+#' @return \code{.std.link.llabels} returns a modified version of \code{misc}
+#'   with the appropriate information included corresponding to the information in \code{fam}
 #' @export
 .std.link.labels = function(fam, misc) {
     if (is.null(fam) || !is.list(fam))
@@ -812,6 +887,8 @@ recover_data.svyglm = function(object, data = NULL, ...) {
         misc$inv.lbl = "prob"
     else if (length(grep("poisson", fam$family)) == 1)
         misc$inv.lbl = "rate"
+    if(length(grep("gaussian", fam$family)) == 0)
+        misc$sigma = NA
     misc
 }
 
