@@ -19,6 +19,10 @@
 #    <http://www.gnu.org/licenses/>.                                         #
 ##############################################################################
 
+# summary of a summary - just pass it thru
+#' @export
+summary.summary_emm = function(object, ...) object
+
 # confint and test methods
 
 # confint method
@@ -145,13 +149,15 @@ test.emmGrid = function(object, null = 0,
             if (inherits(F, "try-error"))
                 c(df1 = r, df2 = NA,  F.ratio = NA, p.value = NA, note = 1)
             else {
-                df2 = min(apply(tQ, 1, function(.) object@dffun(., object@dfargs)))
-                if (is.na(df2))
-                    p.value = suppressWarnings(pchisq(F*r, r, lower.tail = FALSE))
-                else
-                    p.value = suppressWarnings(pf(F, r, df2, lower.tail = FALSE))
+                if(is.null(df2 <- object@misc$df))
+                    df2 = min(apply(tQ, 1, function(.) object@dffun(., object@dfargs)))
+                if (is.na(df2)) 
+                    df2 = Inf
+                p.value = suppressWarnings(pf(F, r, df2, lower.tail = FALSE))
                 rtn = c(round(c(df1 = r, df2 = df2), 2), 
                         F.ratio = round(F, 3), p.value = p.value, note = rrflag)
+                # Note: Following will screw-up joint_tests if some df2's are finite and others infinite
+                if (is.infinite(df2)) rtn = c(rtn[1:3], Chisq = round(F*r, 3), rtn[4:5])
                 attr(rtn, "L") = tQ.all
                 rtn
             }
@@ -235,7 +241,8 @@ test.emmGrid = function(object, null = 0,
 #'   preceding rows.
 #'   
 #'   The returned object also includes an \code{"est.fcns"} attribute, which is a
-#'   named list containing the linear functions associated with each joint test. 
+#'   named list containing the linear functions associated with each joint test.
+#'   Each row of these is standardized to have length 1. 
 #'   No estimable functions are included for confounded effects.
 #'   
 #' @section Dealing with covariates:
@@ -305,9 +312,9 @@ test.emmGrid = function(object, null = 0,
 #' ## ... then tests depend on where we center things
 #' 
 #' # Center around the mean
-#' joint_tests(toy.cov, cov.keep = 0, cov.reduce = meanint)
+#' joint_tests(toy.cov, cov.keep = 0, cov.reduce = make.meanint(delta = 1))
 #' # Center around zero (like SAS's results for toy.cov)
-#' joint_tests(toy.cov, cov.keep = 0, cov.reduce = symmint(0))
+#' joint_tests(toy.cov, cov.keep = 0, cov.reduce = make.symmint(ctr = 0, delta = 1))
 #' # Center around 0.5 (like SAS's results for toy.fac)
 #' joint_tests(toy.cov, cov.keep = 0, cov.reduce = range)
 #' 
@@ -323,7 +330,7 @@ test.emmGrid = function(object, null = 0,
 #' 
 joint_tests = function(object, by = NULL, show0df = FALSE, 
                        showconf = TRUE,
-                       cov.reduce = meanint, ...) {
+                       cov.reduce = make.meanint(1), ...) {
     
     # hidden defaults for contrast methods and which basis to use for all contrasts
     use.contr = (function(use.contr = c("consec", "consec"), ...) use.contr)(...)
@@ -372,8 +379,9 @@ joint_tests = function(object, by = NULL, show0df = FALSE,
                     emm = emmeans(object, these, by = by, ...)
                     tst = test(contrast(emm, interaction = use.contr[1], by = union(by, nesters)), 
                                by = by, joint = TRUE, status = TRUE)
+                    ef = attr(tst, "est.fcns") # get this before we subset the results
+                    tst = tst[names(tst) != "Chisq"]   # could have some with Chisq and some without
                     mt = paste(these, collapse = ":")
-                    ef = attr(tst, "est.fcns")
                     if (length(ef) > 1)
                         ef = list(ef)
                     names(ef) = mt
@@ -381,7 +389,7 @@ joint_tests = function(object, by = NULL, show0df = FALSE,
                     ef.ord <<- c(ef.ord, k)
                     tst = cbind(ord = k, `model term` = mt, tst)
                     result = rbind(result, tst)
-                 }
+                }
             }
             last = max(match(these, facs))
         }
@@ -393,6 +401,12 @@ joint_tests = function(object, by = NULL, show0df = FALSE,
         result
     }
     result = suppressMessages(do.test(character(0), facs, NULL, ...))
+    if (all(is.infinite(result$df2) | is.na(result$df2))) {
+        w = which(names(result) == "F.ratio")
+        result = cbind(result[, 1:w], Chisq = result$F.ratio * result$df1, 
+                       result[, (w+1):ncol(result)])
+    }
+    
     
     ## look at as-yet-unexplained effects
     if (showconf) {
@@ -468,25 +482,40 @@ joint_tests = function(object, by = NULL, show0df = FALSE,
 #' @rdname joint_tests
 #' @order 6
 #'
-#' @param x,ctr arguments for \code{meanint} and \code{symmint}
+#' @param delta,ctr arguments for \code{make.meanint} and \code{make.symmint}
 #'
-#' @return \code{meanint} returns \code{mean(c) + c(-1, 1)}. 
-#'         \code{symmint(ctr)} returns a function that returns \code{ctr + c(-1, 1)}.
+#' @return \code{make.meanint} returns the function 
+#' \code{function(x) mean(x) + delta * c(-1, 1)},
+#'   and \code{make.symmint(ctr, delta)} returns the function
+#' \code{function(x) ctr + delta * c(-1, 1)}
+#'         (which does not depend on \code{x}).
+#'         The cases with \code{delta = 1}, \code{meanint = make.meanint(1)} 
+#'         and \code{symmint(ctr) = make.symmint(ctr, 1)}
+#'         are retained for back-compatibility reasons.
 #'         These functions are available primarily for use with \code{cov.reduce}.
 #' @export
-meanint = function(x) { mean(x) + c(-1, 1) }
+make.meanint = function(delta) 
+    function(x) mean(x) + delta * c(-1, 1)
+
+#' @rdname joint_tests
+#' @order 7
+#' @param x argument for \code{meanint} and \code{symmint}
+#' @export
+meanint = function(x) mean(x) + c(-1, 1)
+
 
-# # zero pm 1
-# #' @rdname joint_tests
-# #' @order 7
-# #' @export
-# zeroint = function(x) { c(-1, 1) }
- 
 # const pm 1
 #' @rdname joint_tests
 #' @order 8
 #' @export
-symmint = function(ctr) {
-    return(function(x) ctr + c(-1, 1)) 
+make.symmint = function(ctr, delta) {
+    function(x) ctr + delta * c(-1, 1) 
 }
 
+#' @rdname joint_tests
+#' @order 9
+#' @export
+symmint = function(ctr) 
+    make.symmint(ctr, 1)
+
+
